/*
 * UKUI Intelligent Data Manager
 *
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: Yue Lan <lanyue@kylinos.cn>
 *
 */

#include "windowhelper.h"

#include <QWindow>
#include <KWindowSystem>
#include <QX11Info>
#include <QMenu>
#include <QApplication>

#include <QDebug>

#include <KWindowEffects>
#include <kwindowsystem_version.h>

#include <xcb/xcb.h>
#include <netwm.h>
#include <xatom-helper.h>
#include <kysdk/applications/ukuistylehelper/ukui-decoration-manager.h>

#include <kaboutdialog.h>

QString getCurrentVersion()
{
    FILE *pp = NULL;
    char *line = NULL;
    size_t len = 0;
    ssize_t read;
    char *q = NULL;
    QString version = QObject::tr("Unknown");

    pp = popen("dpkg -l ukui-intelligent-data-manager", "r");
    if(NULL == pp)
        return version;

    while((read = getline(&line, &len, pp)) != -1){
        q = strrchr(line, '\n');
        *q = '\0';

        QString content = line;
        QStringList list = content.split(" ");

        list.removeAll("");

        if (list.size() >= 3)
            version = list.at(2);
    }

    free(line);
    pclose(pp);
    return version;
}

WindowHelper::WindowHelper(QWindow *parent) : QObject(parent)
{
    rootWindow = parent;
    isX11 = KWindowSystem::isPlatformX11();
    isWayland = KWindowSystem::isPlatformWayland();
}

void WindowHelper::handleSystemMoveRequest(QPointF pos)
{
    auto window = qobject_cast<QWindow *>(sender());

    qreal dpiRatio = window->devicePixelRatio();
    int nativeX = pos.x() * dpiRatio;
    int nativeY = pos.y() * dpiRatio;
    if (KWindowSystem::isPlatformX11()) {
        auto connection = QX11Info::connection();
        xcb_ungrab_pointer(connection, XCB_TIME_CURRENT_TIME);
        NETRootInfo(connection, NET::WMMoveResize).moveResizeRequest(window->winId(), nativeX, nativeY, NET::Move);

        xcb_button_release_event_t* event = new xcb_button_release_event_t;
        memset(event, 0x00, sizeof(xcb_button_release_event_t));
        event->response_type = XCB_BUTTON_RELEASE;
        event->event = window->winId();
        event->time = QX11Info::getTimestamp();
        event->same_screen = 1;
        event->root = QX11Info::appRootWindow();
        event->root_x = nativeX;
        event->root_y = nativeY;
        event->event_x = 0;
        event->event_y = 0;
        event->child = 0;
        event->state = 0;
        event->detail = XCB_BUTTON_INDEX_1;

        xcb_send_event(connection, false, window->winId(), XCB_EVENT_MASK_BUTTON_RELEASE, (char *) event);
        delete event;

        xcb_flush(connection);
    } else if (KWindowSystem::isPlatformWayland()) {
#if (QT_VERSION >= QT_VERSION_CHECK(5,15,0))
        window->startSystemMove();
#else
        if (UKUIDecorationManager::getInstance()->supportUKUIDecoration()) {
            UKUIDecorationManager::getInstance()->moveWindow(rootWindow);
        }
#endif
    }

    qDebug() << "movement" << pos;
}

void WindowHelper::handleOptionMenuRequest(QPointF pos)
{
    QMenu menu;
    auto helpAction = menu.addAction(tr("Help"));
    auto aboutAction = menu.addAction(tr("About"));
    auto quitAction = menu.addAction(tr("Quit"));
    connect(aboutAction, &QAction::triggered, getRootWindow(), [=]{
        kdk::KAboutDialog d;
        d.setAppIcon(getRootWindow()->icon());
        d.setAppName(getRootWindow()->title());
        d.setAppSupport(tr("Service & Support: ") + "<a href=\"mailto://support@kylinos.cn\" style=\"color:"
                        + qApp->palette().color(QPalette::ButtonText).name(QColor::HexArgb)
                        + ";\">support@kylinos.cn</a><br/>");
        d.setAppVersion(QString(tr("Version: %1")).arg(getCurrentVersion()));
        d.exec();
    });
    connect(quitAction, &QAction::triggered, getRootWindow(), &QWindow::close);
    menu.exec(pos.toPoint());
}

void WindowHelper::activeWindow()
{
    qDebug() << rootWindow << rootWindow->winId();

    enableBlur();
    removeHeaderBarX11();
    rootWindow->show();
    // fix wayland active window
    KWindowSystem::forceActiveWindow(rootWindow->winId());
    removeHeaderBarWayland();
}

void WindowHelper::enableBlur()
{
#if KWINDOWSYSTEM_VERSION <= QT_VERSION_CHECK (5, 82, 0)
    KWindowEffects::enableBlurBehind(rootWindow->winId(), true);
#else
    KWindowEffects::enableBlurBehind(rootWindow, true);
#endif
}

void WindowHelper::removeHeaderBarX11()
{
    if (!isX11)
        return;
    MotifWmHints hints;
    hints.flags = MWM_HINTS_FUNCTIONS|MWM_HINTS_DECORATIONS;
    hints.functions = MWM_FUNC_ALL;
    hints.decorations = MWM_DECOR_BORDER;
    XAtomHelper::getInstance()->setWindowMotifHint(rootWindow->winId(), hints);
}

void WindowHelper::removeHeaderBarWayland()
{
    if (!isWayland)
        return;
    if (UKUIDecorationManager::getInstance()->supportUKUIDecoration()) {
        UKUIDecorationManager::getInstance()->removeHeaderBar(rootWindow);
    } else {
        rootWindow->setFlag(Qt::FramelessWindowHint);
    }
}

QWindow *WindowHelper::getRootWindow() const
{
    return rootWindow;
}
